package database

import (
	"database/sql"

	_ "github.com/golang-migrate/migrate/source/file"
)

type Database struct {
	db *sql.DB
}

func InitDb(dbLocation string) (*Database, error) {
	ddb := Database{}

	db, err := sql.Open("sqlite3", "file:"+dbLocation+"?loc=auto")
	if err != nil {
		return &ddb, err
	}

	db.SetConnMaxLifetime(0)
	db.SetMaxOpenConns(5)
	db.SetMaxIdleConns(2)

	err = db.Ping()
	if err != nil {
		return &ddb, err
	}

	ddb.db = db

	return ddb.runMigrations()
}

func (db *Database) runMigrations() (*Database, error) {
	// Hacked up af - Rerunnable
	_, err := db.db.Exec("CREATE TABLE IF NOT EXISTS `quotes` " +
		"(`id` INTEGER PRIMARY KEY AUTOINCREMENT, `user_id` TEXT, `room_id` TEXT, `timestamp` DATETIME, `viewed` INT DEFAULT 0, `quote` TEXT)")
	if err != nil {
		return db, err
	}

	_, err = db.db.Exec("CREATE TABLE IF NOT EXISTS `pics` " +
		"(`id` INTEGER PRIMARY KEY AUTOINCREMENT, `filename` TEXT, `viewed` INT DEFAULT 0)")
	if err != nil {
		return db, err
	}

	_, err = db.db.Exec("CREATE TABLE IF NOT EXISTS `counter` " +
		"(`id` INTEGER PRIMARY KEY AUTOINCREMENT, `key` TEXT, `timestamp` DATETIME)")
	if err != nil {
		return db, err
	}

	return db, nil
}

// CloseDbConn - Closes DB connection
func (db *Database) CloseDbConn() {
	db.db.Close()
}
