package database

import (
	"database/sql"
	"fmt"
	"log"
	"strings"
	"time"

	"maunium.net/go/mautrix/id"
)

// GetAnyRandomQuote
func (db *Database) GetAllQuotes(last int) ([]string, error) {
	lines := []string{}

	row, err := db.db.Query("SELECT `quote`, `user_id`, `id`, `viewed` FROM `quotes` ORDER BY `id` DESC LIMIT ?", last)
	if err != nil {
		return lines, err
	}

	defer row.Close()
	for row.Next() {
		id := int64(0)
		user := ""
		quote := ""
		viewed := int64(0)

		row.Scan(&quote, &user, &id, &viewed)

		lines = append(lines, fmt.Sprintf(
			"(#%d - %s) %s (%d views)",
			id,
			user,
			quote,
			viewed,
		))
	}

	return lines, nil
}

// GetAnyRandomQuote
func (db *Database) GetAnyRandomQuote(search string) (string, string, string, int64, error) {
	min := db.getMinViewCount()
	quote := ""
	user := ""
	room := ""
	id := int64(0)
	var row *sql.Rows
	var err error
	if search != "" {
		row, err = db.db.Query(
			fmt.Sprintf(
				"SELECT `quote`, `user_id`, `room_id`, `id` FROM `quotes` "+
					" WHERE LOWER(`quote`) LIKE \"%s\" ORDER BY RANDOM() LIMIT 1",
				"%"+strings.ReplaceAll(strings.ToLower(strings.ReplaceAll(search, "\"", "")), " ", "%")+"%",
			),
		)
	} else {
		row, err = db.db.Query("SELECT `quote`, `user_id`, `room_id`, `id` FROM `quotes` WHERE `viewed` = ? ORDER BY RANDOM() LIMIT 1", min)
	}
	if err == sql.ErrNoRows {
		return quote, "", "", id, nil
	} else if err != nil {
		return quote, "", "", id, err
	}

	defer row.Close()
	for row.Next() {
		row.Scan(&quote, &user, &room, &id)
	}

	// Only increment if not searching
	if search == "" {
		err = db.incrementViewCount(id)
		if err != nil {
			log.Printf("Error incrementing views: %s", err.Error())
		}
	}

	return quote, user, room, id, nil
}

// StoreQuote
func (db *Database) StoreQuote(roomID id.RoomID, userID id.UserID, quote string) error {
	viewedCount := db.getMinViewCount()
	_, err := db.db.Exec(
		"INSERT INTO `quotes` (`user_id`, `room_id`, `timestamp`, `quote`, `viewed`) VALUES (?,?,?,?, ?)",
		userID.String(), roomID.String(), time.Now().Unix(), quote, viewedCount)
	return err
}

func (db *Database) DeleteQuote(id string) (int64, error) {
	res, _ := db.db.Exec("DELETE FROM `quotes` WHERE `id` = ?", id)
	return res.RowsAffected()
}

func (db *Database) getMinViewCount() int64 {
	var count int64
	db.db.QueryRow("SELECT MIN(`viewed`) FROM `quotes`").Scan(&count)
	return count
}

func (db *Database) getMaxViewCount() int64 {
	var count int64
	db.db.QueryRow("SELECT MAX(`viewed`) FROM `quotes`").Scan(&count)
	return count
}

func (db *Database) incrementViewCount(id int64) error {
	_, err := db.db.Exec("UPDATE `quotes` SET `viewed` = `viewed` + 1 WHERE `id` = ?", id)
	return err
}

func (db *Database) GetStatsByAuthor() ([]string, error) {
	lines := []string{}

	row, err := db.db.Query("SELECT `user_id`, COUNT(*) FROM `quotes` GROUP BY `user_id` ORDER BY COUNT(*) DESC")
	if err != nil {
		return lines, err
	}

	defer row.Close()
	for row.Next() {
		user := ""
		count := int64(0)

		row.Scan(&user, &count)

		lines = append(lines, fmt.Sprintf(
			"%s - %d quotes",
			user,
			count,
		))
	}

	return lines, nil
}
