package gomatrixbot

import (
	"context"
	"fmt"
	"math/rand"
	"os"
	"strings"
	"time"

	"maunium.net/go/mautrix/event"
	"maunium.net/go/mautrix/id"
)

type Server struct {
	mtrx *MtrxClient
}

var COMMANDS = []string{
	"🤠 react to faceswap iamge",
	"🍌 react to upload to is.probablya.bot",
	"📝 react to quote message",
	"🐫 react to save picture for future quote usage",
	"☝️ react to set room topic",
	"📷️ react to save picture for future lulz",
	"",
	".counter - List all counters",
	".counter <key> - Get a counter for a key",
	".counterreset <key> - Reset a counter for a key",
	".r <?quote>- Returns a random quote (can search quotes)",
	".rb <?quote> - Returns a random quote on a random bird pic (with optional search)",
	".remindme <time - 1w1h> <reminder> - Triggers a reminder",
	".imdb <search> - Search IMDB",
	".gif <search> - Get a gif from tenor",
	".giphy <search> - Get a gif from giphy",
	".metar <ICAO> - Returns current METAR for airport",
	".taf <ICAO> - Returns TAF for airport",
	".ud <word> - Urban Dictionary search",
	".vid <weblink> - yt-dlp a video",
	".weather <location> - Get weather",
	".wiki <search> - Search Wikipedia",
	".xkcd <search> - Search XKCD",
	"",
	".stats - Quote stats",
	".top - Top saved images",
	".quake - Returns geonet stuff",
	".delquote <quote ID> - Deltes a saved quote",
	".ping - Check if bot is alive",
	".flip - flips the last message",
}

func (mtrx *MtrxClient) handleReaction(ctx context.Context, evt *event.Event) {
	switch evt.Content.AsReaction().GetRelatesTo().Key {
	case "🤠":
		if val, ok := mtrx.savedFiles[evt.Content.AsReaction().GetRelatesTo().EventID]; ok {
			mtrx.swapFace(ctx, evt.RoomID, val)
		} else {
			mtrx.sendMessage(ctx, evt.RoomID, "Unable to find image to upload")
		}
		return
	case "🍌":
		if val, ok := mtrx.savedFiles[evt.Content.AsReaction().GetRelatesTo().EventID]; ok {
			mtrx.uploadToSnac(ctx, evt.RoomID, val)
			return
		} else if val, ok := mtrx.events[evt.Content.AsReaction().GetRelatesTo().EventID]; ok {
			mtrx.uploadTxtToSnac(ctx, evt.RoomID, val)
		}
	case "📝":
		mtrx.quoteThis(ctx, evt)
	case "🐫":
		// Save to reaction pic
		if val, ok := mtrx.savedFiles[evt.Content.AsReaction().GetRelatesTo().EventID]; ok {
			mtrx.saveSourcePic(ctx, val, evt.RoomID)
		}
	case "☝️":
		mtrx.setTopic(ctx, evt)
	case "📷️":
		fallthrough
	case "📸":
		// Save to disk!
		if val, ok := mtrx.savedFiles[evt.Content.AsReaction().GetRelatesTo().EventID]; ok {
			mtrx.saveTheGreat(ctx, val, evt.RoomID)
		}
		// case "😶":
		// 	if val, ok := mtrx.savedFiles[evt.Content.AsReaction().GetRelatesTo().EventID]; ok {
		// 		mtrx.interpretImage(ctx, evt.RoomID, val)
		// 	} else {
		// 		mtrx.sendMessage(ctx, evt.RoomID, "Unable to find image to describe")
		// 	}
	}
}

func (mtrx *MtrxClient) handleCommand(ctx context.Context, evt *event.Event) {
	// Split the command
	args := strings.Split(evt.Content.AsMessage().Body, " ")
	if evt.Content.AsMessage().NewContent != nil {
		args = strings.Split(evt.Content.AsMessage().NewContent.Body, " ")
	}

	switch strings.ToLower(args[0]) {
	case ".":
		return
	// case ".askgary":
	// 	if len(args) < 2 {
	// 		mtrx.sendMessage(ctx, evt.RoomID, "What do you want to ask?")
	// 	} else {
	// 		mtrx.interpretText(ctx, evt.RoomID, strings.Join(args[1:], " "))
	// 	}
	case ".help":
		mtrx.sendMessage(ctx, evt.RoomID, mtrx.getHelp())
		return
	case ".die":
		mtrx.sendMessage(ctx, evt.RoomID, "AM DYING NOW")
		os.Exit(1)
		return
	case ".remindme":
		if len(args) < 3 {
			mtrx.sendMessage(ctx, evt.RoomID, "Usage: .remindme <time> <message>")
			return
		}
		mtrx.remindMe(ctx, evt.Sender, evt.RoomID, args[1], strings.Join(args[2:], " "))
		return
	case ".update":
		err := mtrx.UpdatePics()
		if err != nil {
			mtrx.sendMessage(ctx, evt.RoomID, err.Error())
		}
		return
	case ".counter":
		if len(args) < 2 {
			mtrx.getAllCounters(ctx, evt.RoomID)
			return
		} else {
			mtrx.getCounterFor(ctx, evt.RoomID, strings.Join(args[1:], " "))
			return
		}
	case ".counterreset":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "Usage: .counterreset <key>")
			return
		} else {
			mtrx.updateCounterFor(ctx, evt.RoomID, strings.Join(args[1:], " "))
			return
		}
	case ".counterdel":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "Usage: .counterdel <key>")
			return
		} else {
			mtrx.deleteCounterFor(ctx, evt.RoomID, strings.Join(args[1:], " "))
			return
		}
	case ".reloadallpics":
		mtrx.db.DeleteAllPics()
		mtrx.pics = make(map[int64]string, 0)
		mtrx.UpdatePics()
		mtrx.sendMessage(ctx, evt.RoomID, "Reset all pictures")
		return
	case ".resetpiccount":
		mtrx.db.ResetPics()
		mtrx.sendMessage(ctx, evt.RoomID, "Reset all image viewed counts")
		return
	case ".top":
		mtrx.postTheGreats(ctx, evt.RoomID)
		return
	// case ".fuckme":
	// 	mtrx.purge(ctx, evt.RoomID)
	// 	return
	case ".weather":
		mtrx.fetchWeather(ctx, evt.RoomID, strings.Join(args[1:], " "))
	case ".forecast":
		mtrx.fetchForecast(ctx, evt.RoomID, strings.Join(args[1:], " "))
	case ".xkcd":
		mtrx.searchXKCD(ctx, evt.RoomID, strings.Join(args[1:], " "))
		return
	case ".ud":
		mtrx.sendMessage(ctx, evt.RoomID, mtrx.getUrbanDictionary(strings.Join(args[1:], " ")))
	case ".vid":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "Please provide a video link")
		} else {
			mtrx.pullVideo(ctx, evt.RoomID, args[1])
		}
	case ".r":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, mtrx.getAnyRandomQuote(false))
		} else {
			mtrx.sendMessage(ctx, evt.RoomID, mtrx.getAnyRandomQuoteSearch(false, strings.Join(args[1:], " ")))
		}
		return
	case ".rb":
		if len(args) < 2 {
			mtrx.postBirbWithText(ctx, evt, mtrx.getAnyRandomQuote(true), "")
		} else {
			mtrx.postBirbWithText(ctx, evt, mtrx.getAnyRandomQuoteSearch(true, strings.Join(args[1:], " ")), "")
		}
		return
	case ".delquote":
		rows, err := mtrx.deleteQuote(args[1])
		if err != nil {
			mtrx.sendMessage(ctx, evt.RoomID, err.Error())
		} else {
			mtrx.sendMessage(ctx, evt.RoomID, fmt.Sprintf("Deleted %d quote(s)", rows))
		}
		return
	case ".quake":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "usage: .quake <intensity 1-5>")
			return
		}
		mtrx.checkForQuakes(ctx, evt.RoomID, args[1])
		return
	case ".imdb":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "Give me a movie/tv title or something pls?")
			return
		}

		mtrx.searchImdb(ctx, evt.RoomID, strings.Join(args[1:], " "))
		return
	case ".metar":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "Please provide a station code")
			return
		}
		mtrx.sendMetar(ctx, evt.RoomID, args[1])
	case ".taf":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "Please provide a station code")
			return
		}
		mtrx.sendTaf(ctx, evt.RoomID, args[1])
	case ".wiki":
		if len(args) < 2 {
			mtrx.sendMessage(ctx, evt.RoomID, "Please provide a search term")
			return
		}
		mtrx.searchWiki(ctx, evt.RoomID, strings.Join(args[1:], " "))
	case ".ping":
		mtrx.sendMessage(ctx, evt.RoomID, "pong")
		return
	case ".gif":
		mtrx.PostTenorGif(ctx, evt.RoomID, args[1:], 5)
	case ".giphy":
		mtrx.PostGiphyGif(ctx, evt.RoomID, args[1:])
	case ".flip":
		mtrx.FlipText(ctx, evt.RoomID)
	case ".stats":
		quotes, err := mtrx.db.GetStatsByAuthor()
		if err != nil {
			mtrx.sendMessage(ctx, evt.RoomID, "Err: "+err.Error())
		} else {
			i := 0
			mtrx.sendNotice(ctx, evt.RoomID, "Quote Stats:\n")
			tmpLines := []string{}
			for _, v := range quotes {
				if i > 10 {
					mtrx.sendNotice(ctx, evt.RoomID, strings.Join(tmpLines, "\n"))
					i = 0
					tmpLines = []string{}
				}
				tmpLines = append(tmpLines, v)
				i++
			}
			if i > 0 {
				mtrx.sendNotice(ctx, evt.RoomID, strings.Join(tmpLines, "\n"))
			}
		}
	case ".spam":
		quotes, err := mtrx.db.GetAllQuotes(10)
		if err != nil {
			mtrx.sendMessage(ctx, evt.RoomID, "Err: "+err.Error())
		} else {
			i := 0
			mtrx.sendMessage(ctx, evt.RoomID, "All quotes:\n")
			tmpLines := []string{}
			for _, v := range quotes {
				if i > 10 {
					mtrx.sendMessage(ctx, evt.RoomID, strings.Join(tmpLines, "\n"))
					i = 0
					tmpLines = []string{}
				}
				tmpLines = append(tmpLines, v)
				i++
			}
			if i > 0 {
				mtrx.sendMessage(ctx, evt.RoomID, strings.Join(tmpLines, "\n"))
			}
		}
	default:
		if strings.HasPrefix(args[0], "..") {
			return
		}
		filesearch := strings.ReplaceAll(args[0], ".", "")

		// lookup filename from quote, shuffle each time
		matches := make(map[int64]string)
		keys := make([]int64, 0)
		for k, v := range mtrx.pics {
			if strings.Contains(strings.ToLower(strings.Split(v, ".")[0]), strings.ToLower(filesearch)) {
				matches[k] = v
				keys = append(keys, k)
			}
		}

		// Shuffle key
		if len(matches) > 0 {
			filename := matches[keys[rand.Intn(len(keys))]]
			mtrx.postBirbWithText(ctx, evt, mtrx.getAnyRandomQuoteSearch(true, strings.Join(args[1:], " ")), filename)
		}
	}
}

func (mtrx *MtrxClient) purgeOutput() string {
	count := 0
	files, err := os.ReadDir(OUTPUT_PATH)
	if err != nil {
		return "Failed to list files"
	}

	for _, file := range files {
		if file.IsDir() || strings.HasPrefix(file.Name(), ".") {
			continue
		}

		err = os.Remove(OUTPUT_PATH + "/" + file.Name())
		if err == nil {
			count = count + 1
		}
	}

	// Clear recents
	mtrx.savedFiles = make(map[id.EventID]string, 0)
	mtrx.events = make(map[id.EventID]string, 0)
	mtrx.quoteCache = make(map[id.EventID]*Quote, 0)

	return fmt.Sprintf("Deleted %d files", count)
}

func (mtrx *MtrxClient) handleImage(ctx context.Context, evt *event.Event) {
	mxc := ""
	if evt.Content.AsMessage().File != nil {
		mxc = string(evt.Content.AsMessage().File.URL)
	} else {
		mxc = string(evt.Content.AsMessage().URL)
	}

	// Split mxc
	mxc = strings.ReplaceAll(mxc, "mxc://", "")
	mxcParts := strings.Split(mxc, "/")
	if len(mxcParts) == 2 {
		// Download the file
		remoteFile := fmt.Sprintf(
			"%s/_matrix/media/v3/download/%s/%s?allow_redirect=true",
			os.Getenv("MATRIX_HOST"),
			mxcParts[0],
			mxcParts[1],
		)

		rand := time.Now().UnixNano()
		fileType := ""
		if evt.Content.AsMessage().Info.MimeType == "image/png" {
			fileType = "png"
		} else if evt.Content.AsMessage().Info.MimeType == "image/jpeg" {
			fileType = "jpg"
		} else if evt.Content.AsMessage().Info.MimeType == "video/mp4" {
			fileType = "mp4"
		} else {
			return
		}

		// GetFile content
		content, err := mtrx.getFileContent(remoteFile)
		if err != nil {
			mtrx.c.Log.Err(err).Msg("Failed to download file")
			return
		}

		// Decrypt the file
		if evt.Content.AsMessage().File != nil {
			err = evt.Content.AsMessage().File.DecryptInPlace(content)
			if err != nil {
				mtrx.c.Log.Err(err).Msg("Failed to decrypt file")
				return
			}
		}

		// Copy locally
		localFile := fmt.Sprintf("%s/old_%d.%s", OUTPUT_PATH, rand, fileType)
		// newLocalFile := fmt.Sprintf("%s/new_%d.%s", OUTPUT_PATH, rand, fileType)

		f, err := os.Create(localFile)
		if err != nil {
			mtrx.c.Log.Err(err).Msg("Failed to create file")
			return
		}
		_, err = f.WriteString(string(content))
		if err != nil {
			mtrx.c.Log.Err(err).Msg("Failed to write file")
			return
		}
		f.Close()
		mtrx.savedFiles[evt.ID] = localFile
	}
}

func (mtrx *MtrxClient) getHelp() string {
	return strings.Join(COMMANDS, "\n")
}
