package gomatrixbot

import (
	"context"
	"fmt"
	"strings"

	"maunium.net/go/mautrix/event"
	"maunium.net/go/mautrix/id"
)

type Quote struct {
	Quote string
	User  id.UserID
}

func (mtrx *MtrxClient) quoteThis(ctx context.Context, evt *event.Event) error {
	if val, ok := mtrx.quoteCache[evt.Content.AsReaction().GetRelatesTo().EventID]; ok {
		quote := val.Quote
		delete(mtrx.quoteCache, evt.Content.AsReaction().GetRelatesTo().EventID)

		// If a reply to a quote is a quote, delete the quote lolhax
		if strings.HasPrefix(val.Quote, ">") && strings.Contains(val.Quote, "\n\n") {
			parts := strings.Split(val.Quote, "\n\n")
			quote = strings.Join(parts[1:], "\n")
		}
		return mtrx.db.StoreQuote(evt.RoomID, val.User, quote)
	}

	return nil
}

func (mtrx *MtrxClient) getAnyRandomQuoteSearch(msgOnly bool, search string) string {
	quote, user, _, id, err := mtrx.db.GetAnyRandomQuote(search)
	if err != nil {
		return err.Error()
	}

	if quote == "" {
		return search
	}

	if msgOnly {
		return quote
	}

	return fmt.Sprintf(
		"Quote from %s (#%d)\n%s",
		user,
		id,
		quote,
	)
}

func (mtrx *MtrxClient) getAnyRandomQuote(msgOnly bool) string {
	quote, user, _, id, err := mtrx.db.GetAnyRandomQuote("")
	if err != nil {
		return err.Error()
	}

	if quote == "" {
		return "No quotes found??"
	}

	if msgOnly {
		return quote
	}

	return fmt.Sprintf(
		"Quote from %s (#%d)\n%s",
		user,
		id,
		quote,
	)
}

func (mtrx *MtrxClient) deleteQuote(quoteId string) (int64, error) {
	return mtrx.db.DeleteQuote(quoteId)
}
