package gomatrixbot

import (
	"context"
	"regexp"
	"strings"
	"time"

	"maunium.net/go/mautrix/id"
)

func (m *MtrxClient) remindMe(ctx context.Context, userID id.UserID, roomID id.RoomID, duration, message string) {
	// Parse the duration string into a time.Duration
	d, err := ParseDuration(duration)
	// d, err := time.ParseDuration(duration)
	if err != nil {
		m.sendMessage(ctx, roomID, "Failed to parse duration: "+err.Error())
		return
	}

	// Create a timer that will send the message after the duration has passed
	timer := time.NewTimer(d)
	go func() {
		<-timer.C
		m.sendMessage(ctx, roomID, string(userID)+" you asked me to remind you: "+message)
	}()

	m.sendMessage(ctx, roomID, "Ok, I will remind you in "+duration)
}

// ParseDuration parses a duration string.
// examples: "10d", "-1.5w" or "3Y4M5d".
// Add time units are "d"="D", "w"="W", "M", "y"="Y".
func ParseDuration(s string) (time.Duration, error) {
	neg := false
	if len(s) > 0 && s[0] == '-' {
		neg = true
		s = s[1:]
	}

	re := regexp.MustCompile(`(\d*\.\d+|\d+)[^\d]*`)
	unitMap := map[string]time.Duration{
		"d": 24,
		"D": 24,
		"w": 7 * 24,
		"W": 7 * 24,
		"M": 30 * 24,
		"y": 365 * 24,
		"Y": 365 * 24,
	}

	strs := re.FindAllString(s, -1)
	var sumDur time.Duration
	for _, str := range strs {
		var _hours time.Duration = 1
		for unit, hours := range unitMap {
			if strings.Contains(str, unit) {
				str = strings.ReplaceAll(str, unit, "h")
				_hours = hours
				break
			}
		}

		dur, err := time.ParseDuration(str)
		if err != nil {
			return 0, err
		}

		sumDur += dur * _hours
	}

	if neg {
		sumDur = -sumDur
	}
	return sumDur, nil
}
