package gomatrixbot

import (
	"context"
	"os"

	"github.com/mattn/go-mastodon"
	"maunium.net/go/mautrix/id"
)

var snacClient *mastodon.Client

// upload Image to snac
func (mtrx *MtrxClient) uploadToSnac(ctx context.Context, roomID id.RoomID, file string) {
	snac := mastodon.NewClient(&mastodon.Config{
		Server:      os.Getenv("SNAC_HOST"),
		AccessToken: os.Getenv("SNAC_ACCESS_TOKEN"),
	})

	atch, err := snac.UploadMedia(ctx, file)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Failed to upload file: "+err.Error())
		return
	}

	atch.Description = "A meme image with some text overlay describing a funny situation"

	toot := &mastodon.Toot{
		Status:     "",
		MediaIDs:   []mastodon.ID{atch.ID},
		Visibility: mastodon.VisibilityPublic,
		Sensitive:  false,
	}

	resp, err := snac.PostStatus(ctx, toot)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Failed to post toot: "+err.Error())
		return
	}

	mtrx.sendMessage(ctx, roomID, resp.URL)
}

// upload Image to snac
func (mtrx *MtrxClient) uploadTxtToSnac(ctx context.Context, roomID id.RoomID, txt string) {
	snac := mastodon.NewClient(&mastodon.Config{
		Server:      os.Getenv("SNAC_HOST"),
		AccessToken: os.Getenv("SNAC_ACCESS_TOKEN"),
	})

	toot := &mastodon.Toot{
		Status:     txt,
		Visibility: mastodon.VisibilityPublic,
		Sensitive:  false,
	}

	resp, err := snac.PostStatus(ctx, toot)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Failed to post toot: "+err.Error())
		return
	}

	mtrx.sendMessage(ctx, roomID, resp.URL)
}
