package gomatrixbot

import (
	"context"
	"encoding/json"
	"fmt"
	"io"
	"math/rand"
	"net/http"
	"net/url"
	"os"
	"strings"
	"time"

	"maunium.net/go/mautrix/event"
	"maunium.net/go/mautrix/id"
)

type TenorResponse struct {
	Results []struct {
		ID                 string `json:"id"`
		Title              string `json:"title"`
		ContentDescription string `json:"content_description"`
		Media              []struct {
			Gif struct {
				URL     string `json:"url"`
				Preview string `json:"preview"`
				Size    int    `json:"size"`
				Dims    []int  `json:"dims"`
			} `json:"gif"`
		} `json:"media"`
		BgColor string `json:"bg_color"`
		Itemurl string `json:"itemurl"`
		URL     string `json:"url"`
	} `json:"results"`
	Next string `json:"next"`
}

func (mtrx *MtrxClient) PostTenorGif(ctx context.Context, roomID id.RoomID, s []string, limit int) {
	apiKey := os.Getenv("TENOR_API_KEY")

	url := fmt.Sprintf(
		"https://g.tenor.com/v1/search?q=%s&key=%s&limit=%d",
		url.QueryEscape(strings.Join(s, " ")),
		apiKey,
		limit,
	)

	mtrx.c.Log.Info().Str("url", url).Msg("Searching for gif")

	resp, err := http.Get(url)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}

	body, err := io.ReadAll(resp.Body)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}

	var tenorResponse TenorResponse
	err = json.Unmarshal(body, &tenorResponse)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}

	if len(tenorResponse.Results) > 0 {
		s := rand.NewSource(time.Now().UnixMicro())
		r := rand.New(s)
		rand := r.Intn(len(tenorResponse.Results))

		// Get
		content, err := mtrx.getFileContent(tenorResponse.Results[rand].Media[0].Gif.URL)
		if err != nil {
			mtrx.sendMessage(ctx, roomID, err.Error())
			return
		}

		media, err := mtrx.c.UploadBytesWithName(ctx, content, "image/gif", tenorResponse.Results[rand].ID+".gif")
		if err != nil {
			mtrx.c.Log.Err(err).Msg("Failed to upload file")
			return
		}
		_, err = mtrx.c.SendMessageEvent(ctx, roomID, event.EventMessage, &event.MessageEventContent{
			MsgType: event.MsgImage,
			Info: &event.FileInfo{
				MimeType: "image/gif",
				Width:    tenorResponse.Results[rand].Media[0].Gif.Dims[0],
				Height:   tenorResponse.Results[rand].Media[0].Gif.Dims[1],
			},
			FileName: tenorResponse.Results[rand].ID + ".gif",
			Body:     "",
			URL:      id.ContentURIString(media.ContentURI.String()),
		})
		if err != nil {
			mtrx.sendMessage(ctx, roomID, err.Error())
		}

		return
	}

	mtrx.sendMessage(ctx, roomID, "No gifs found")

}
